@@ -361,7 +361,7 @@ on_button_press (GooCanvasItem *item,
 
 	  fleur = gdk_cursor_new (GDK_FLEUR);
 	  goo_canvas_pointer_grab (canvas, item,
-				   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+				   GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_RELEASE_MASK,
 				   fleur,
 				   event->time);
 	  gdk_cursor_unref (fleur);
@@ -385,6 +385,25 @@ on_button_press (GooCanvasItem *item,
 }
 
 
+static gboolean
+on_scroll (GooCanvasItem *item,
+	   GooCanvasItem *target,
+	   GdkEventScroll *event,
+	   gpointer data)
+{
+#if 0
+  g_print ("received 'scroll-event' signal\n");
+#endif
+
+  if (event->direction == GDK_SCROLL_UP)
+    goo_canvas_item_scale (item, 1.1, 1.1);
+  else if (event->direction == GDK_SCROLL_DOWN)
+    goo_canvas_item_scale (item, 0.909, 0.909);
+  else return FALSE;
+
+  return TRUE;
+}
+
 static gboolean
 on_button_release (GooCanvasItem *item,
 		   GooCanvasItem *target,
@@ -404,7 +423,6 @@ on_button_release (GooCanvasItem *item,
   return TRUE;
 }
 
-
 static gboolean
 on_background_button_press (GooCanvasItem *item,
 			    GooCanvasItem *target,
@@ -412,6 +430,7 @@ on_background_button_press (GooCanvasItem *item,
 			    gpointer data)
 {
   GooCanvas *canvas;
+  GList *items, *elem;
 
 #if 1
   g_print ("background received 'button-press' signal\n");
@@ -420,6 +439,12 @@ on_background_button_press (GooCanvasItem *item,
   canvas = goo_canvas_item_get_canvas (item);
   output_items_in_area (canvas, event->x_root, event->y_root);
 
+  items = goo_canvas_get_items_at (canvas, event->x_root, event->y_root,
+				   FALSE);
+  for (elem = items; elem; elem = elem->next)
+    g_print ("  clicked items: %p\n", elem->data);
+  g_list_free (items);
+
   return TRUE;
 }
 
@@ -428,11 +453,11 @@ static void
 setup_item_signals (GooCanvasItem *item)
 {
   g_signal_connect (item, "motion_notify_event",
-		    (GtkSignalFunc) on_motion_notify, NULL);
+		    G_CALLBACK (on_motion_notify), NULL);
   g_signal_connect (item, "button_press_event",
-		    (GtkSignalFunc) on_button_press, NULL);
+		    G_CALLBACK (on_button_press), NULL);
   g_signal_connect (item, "button_release_event",
-		    (GtkSignalFunc) on_button_release, NULL);
+		    G_CALLBACK (on_button_release), NULL);
 }
 
 
@@ -466,6 +491,10 @@ create_canvas_primitives ()
 		      "bounds-from-origin", FALSE,
 		      "bounds-padding", 4.0,
 		      "background-color-rgb", 0xC3C3FF,
+ 		      "has-tooltip", TRUE,
+#if 0
+		      "redraw-when-scrolled", TRUE,
+#endif
 		      NULL);
 	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 604, 454);
 
@@ -478,7 +507,7 @@ create_canvas_primitives ()
 	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
-			  (GtkSignalFunc) zoom_changed,
+			  G_CALLBACK (zoom_changed),
 			  canvas);
 	gtk_widget_set_size_request (w, 50, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
@@ -492,7 +521,7 @@ create_canvas_primitives ()
 	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
-			  (GtkSignalFunc) zoom_x_changed,
+			  G_CALLBACK (zoom_x_changed),
 			  canvas);
 	gtk_widget_set_size_request (w, 50, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
@@ -506,7 +535,7 @@ create_canvas_primitives ()
 	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
-			  (GtkSignalFunc) zoom_y_changed,
+			  G_CALLBACK (zoom_y_changed),
 			  canvas);
 	gtk_widget_set_size_request (w, 50, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
@@ -523,7 +552,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	/*gtk_widget_show (w);*/
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) center_toggled,
+			  G_CALLBACK (center_toggled),
 			  canvas);
 
 	/* Move Ellipse */
@@ -532,7 +561,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) move_ellipse_clicked,
+			  G_CALLBACK (move_ellipse_clicked),
 			  canvas);
 
 	/* Animate Ellipse */
@@ -541,7 +570,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) animate_ellipse_clicked,
+			  G_CALLBACK (animate_ellipse_clicked),
 			  canvas);
 	
 	/* Stop Animation */
@@ -550,7 +579,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) stop_animation_clicked,
+			  G_CALLBACK (stop_animation_clicked),
 			  canvas);
 	
 
@@ -561,7 +590,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) write_pdf_clicked,
+			  G_CALLBACK (write_pdf_clicked),
 			  canvas);
 #endif
 
@@ -569,7 +598,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) change_bounds_clicked,
+			  G_CALLBACK (change_bounds_clicked),
 			  canvas);
 
 	hbox = gtk_hbox_new (FALSE, 4);
@@ -586,21 +615,21 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) scroll_to_50_50_clicked,
+			  G_CALLBACK (scroll_to_50_50_clicked),
 			  canvas);
 
 	w = gtk_button_new_with_label("250,250");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) scroll_to_250_250_clicked,
+			  G_CALLBACK (scroll_to_250_250_clicked),
 			  canvas);
 
 	w = gtk_button_new_with_label("500,500");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) scroll_to_500_500_clicked,
+			  G_CALLBACK (scroll_to_500_500_clicked),
 			  canvas);
 
 	/* Scroll anchor */
@@ -614,7 +643,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_NW));
@@ -624,7 +653,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_N));
@@ -634,7 +663,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_NE));
@@ -644,7 +673,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_W));
@@ -654,7 +683,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_CENTER));
@@ -664,7 +693,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_E));
@@ -674,7 +703,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_SW));
@@ -684,7 +713,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_S));
@@ -694,7 +723,7 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_SE));
@@ -714,7 +743,7 @@ create_canvas_primitives ()
 
 #if 0
 	g_signal_connect_after (canvas, "key_press_event",
-				(GtkSignalFunc) key_press,
+				G_CALLBACK (key_press),
 				NULL);
 
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_FOCUS);
@@ -731,12 +760,20 @@ setup_heading (GooCanvasItem *root, char *text, int pos)
   double x = (pos % 3) * 200 + 100;
   double y = (pos / 3) * 150 + 5;
   GooCanvasItem *item;
+  PangoRectangle ink_rect, logical_rect;
 
   item = goo_canvas_text_new (root, text, x, y, -1, GTK_ANCHOR_N,
 			      "font", "Sans 12",
 			      NULL);
   goo_canvas_item_skew_y (item, 30, x, y);
   /*goo_canvas_item_rotate (item, 30, x, y);*/
+
+  goo_canvas_text_get_natural_extents (GOO_CANVAS_TEXT (item),
+				       &ink_rect, &logical_rect);
+  g_print ("Ink Extents: %i,%i %ix%i  Logical: %i,%i %ix%i\n",
+	   ink_rect.x, ink_rect.y, ink_rect.width, ink_rect.height,
+	   logical_rect.x, logical_rect.y,
+	   logical_rect.width, logical_rect.height);
 }
 
 
@@ -816,6 +853,7 @@ setup_rectangles (GooCanvasItem *root)
   item = goo_canvas_rect_new (root, 20, 30, 50, 30,
 			      "stroke-color", "red",
 			      "line-width", 8.0,
+			      "tooltip", "Red stroked rectangle",
 			      NULL);
   setup_item_signals (item);
 
@@ -824,6 +862,7 @@ setup_rectangles (GooCanvasItem *root)
 			      "fill-pattern", pattern,
 			      "stroke-color", "black",
 			      "line-width", 4.0,
+			      "tooltip", "Medium Sea Green stippled rectangle",
 			      NULL);
   cairo_pattern_destroy (pattern);
   setup_item_signals (item);
@@ -831,6 +870,7 @@ setup_rectangles (GooCanvasItem *root)
   item = goo_canvas_rect_new (root, 10, 80, 70, 60,
 			      "fill-color", "steelblue",
 			      /*"fill-pattern", NULL,*/
+			      "tooltip", "Steel Blue rectangle",
 			      NULL);
   setup_item_signals (item);
 
@@ -838,6 +878,7 @@ setup_rectangles (GooCanvasItem *root)
 			      "fill-color-rgba", 0x3cb37180,
 			      "stroke-color", "blue",
 			      "line-width", 2.0,
+			      "tooltip", "Partially transparent rectangle",
 			      NULL);
   setup_item_signals (item);
 
@@ -846,11 +887,13 @@ setup_rectangles (GooCanvasItem *root)
 			      "radius-y", 10.0,
 			      "stroke-color", "yellow",
 			      "fill-color-rgba", 0x3cb3f180,
+			      "tooltip", "Rectangle with rounded corners",
 			      NULL);
   setup_item_signals (item);
 
   item = goo_demo_item_new (root, 30, 20, 50, 30,
 			    "fill-color", "yellow",
+			    "tooltip", "Yellow demo item",
 			    NULL);
   setup_item_signals (item);
 }
@@ -1080,6 +1123,7 @@ setup_polygons (GooCanvasItem *root)
   cairo_pattern_destroy (pattern);
   goo_canvas_points_unref (points);
   setup_item_signals (polyline1);
+  
 
   points = goo_canvas_points_new (14);
   points->coords[0] = 270.0;
@@ -1140,6 +1184,9 @@ make_anchor (GooCanvasItem *root, double x, double y)
 			      NULL);
   setup_item_signals (item);
 
+  g_signal_connect (group, "scroll_event",
+		    G_CALLBACK (on_scroll), NULL);
+
   return group;
 }
 
@@ -1274,8 +1321,9 @@ plant_flower (GooCanvasItem *root, double x, double y, GtkAnchorType anchor)
 
   image = goo_canvas_image_new (root, NULL, x, y,
 			       "pattern", pattern,
-			       "width", w,
-			       "height", h,
+			       "width", w * 1.5,
+			       "height", h * 2,
+				"scale-to-fit", TRUE,
 			       NULL);
   cairo_pattern_destroy (pattern);
   setup_item_signals (image);
@@ -1311,6 +1359,141 @@ setup_images (GooCanvasItem *root)
 }
 
 
+static void
+setup_static_items (GooCanvas *canvas)
+{
+  GooCanvasItem *static_root, *group, *item;
+
+  static_root = goo_canvas_get_static_root_item (canvas);
+
+  /* All static items in one place in the canvas should be placed in the same
+     group. */
+  group = goo_canvas_group_new (static_root, NULL);
+
+  item = goo_canvas_polyline_new_line (group,
+				       40.0, 410.0,
+				       40.0, 330.0,
+				       "stroke-color", "midnightblue",
+				       "line-width", 3.0,
+				       "end-arrow", TRUE,
+				       "arrow-tip-length", 3.0,
+				       "arrow-length", 4.0,
+				       "arrow-width", 3.5,
+				       NULL);
+  setup_item_signals (item);
+
+  item = goo_canvas_polyline_new_line (group,
+				       32.0, 370.0,
+				       48.0, 370.0,
+				       "stroke-color", "midnightblue",
+				       "line-width", 3.0,
+				       NULL);
+  setup_item_signals (item);
+
+  item = goo_canvas_text_new (group, "N", 40, 320, -1, GTK_ANCHOR_S,
+			      "font", "Sans 12",
+			      NULL);
+  setup_item_signals (item);
+}
+
+
+/* This checks that the rgba color properties work properly, i.e. the value
+   written to them is the same when read back out. */
+static void
+test_color_properties (GooCanvasItem *root)
+{
+#if 0
+  GooCanvasItem *item;
+  guint red, green, blue, alpha, rgba_in, rgba_out;
+
+  item = goo_canvas_rect_new (root, 20, 30, 50, 30,
+			      NULL);
+
+  for (red = 0; red < 256; red += 15)
+    {
+      for (green = 0; green < 256; green += 15)
+	{     
+	  for (blue = 0; blue < 256; blue ++)
+	    {     
+	      for (alpha = 0; alpha < 256; alpha += 15)
+		{     
+		  rgba_in = (red << 24) + (green << 16) + (blue << 8) + alpha;
+		  g_object_set (item, "fill_color_rgba", rgba_in, NULL);
+		  rgba_out = 0;
+		  g_object_get (item, "fill_color_rgba", &rgba_out, NULL);
+
+		  if (rgba_in != rgba_out)
+		    g_print ("ERROR: rgba in: %.8X out: %.8X\n", rgba_in, rgba_out);
+		}
+	    }
+	}
+    }
+#endif
+}
+
+static void
+test_simple_transforms (GooCanvasItem *root)
+{
+#if 0
+  GooCanvasItem *item;
+  int rotation, x, y;
+  double scale, x_out, y_out, scale_out, rotation_out, max_error = 0.0000001;
+
+  item = goo_canvas_rect_new (root, 20, 30, 50, 30,
+			      NULL);
+
+  for (rotation = 0; rotation < 360; rotation += 20)
+    {
+      scale = 0.1;
+      while (scale < 10)
+	{
+	  g_print ("Rotation: %i Scale: %g\n", rotation, scale);
+
+	  for (x = -100; x < 100; x += 10)
+	    {
+	      for (y = -100; y < 100; y += 10)
+		{
+		  goo_canvas_item_set_simple_transform (item, x, y, scale,
+							rotation);
+
+		  goo_canvas_item_get_simple_transform (item, &x_out, &y_out,
+							&scale_out,
+							&rotation_out);
+
+		  if (fabs (x_out - x) > max_error
+		      || fabs (y_out - y) > max_error
+		      || fabs (scale_out - scale) > max_error
+		      || fabs (rotation_out - rotation) > max_error)
+		    {
+		      g_print ("IN X:%i Y:%i Scale:%g Rotation:%i -> %g, %g, %g, %g\n", x, y, scale, rotation, x_out, y_out, scale_out, rotation_out);
+		    }
+		}
+	    }
+	  scale += 0.1;
+	}
+    }
+#endif
+}
+
+
+static void
+setup_grids (GooCanvasItem *root)
+{
+  GooCanvasItem *item;
+
+  item = goo_canvas_grid_new (root, 80, 310, 90, 90, 10, 10, 5, 5,
+			      "stroke-color", "yellow",
+			      "fill-color", "pink",
+			      "border-width", 2.0,
+			      "border-color", "red",
+			      "vert-grid-line-color", "lightblue",
+			      "horz-grid-line-width", 1.0,
+			      "vert-grid-line-width", 1.0,
+			      "vert-grid-lines-on-top", TRUE,
+			      NULL);
+}
+
+
 static void
 setup_canvas (GooCanvas *canvas)
 {
@@ -1318,7 +1501,7 @@ setup_canvas (GooCanvas *canvas)
 
   root = goo_canvas_get_root_item (canvas);
   g_signal_connect (root, "button_press_event",
-		    (GtkSignalFunc) on_background_button_press, NULL);
+		    G_CALLBACK (on_background_button_press), NULL);
 
   /* Setup canvas items */
 #if 1
@@ -1330,7 +1513,12 @@ setup_canvas (GooCanvas *canvas)
   setup_texts (root); 
   setup_images (root);
   setup_invisible_texts (root);
+  setup_static_items (canvas);
+  setup_grids (root);
 #endif
+
+  test_color_properties (root);
+  test_simple_transforms (root);
 }
 
 
@@ -1352,7 +1540,7 @@ create_window ()
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size (GTK_WINDOW (window), 640, 600);
   gtk_widget_show (window);
-  g_signal_connect (window, "delete_event", (GtkSignalFunc) on_delete_event,
+  g_signal_connect (window, "delete_event", G_CALLBACK (on_delete_event),
 		    NULL);
 
   notebook = gtk_notebook_new ();
