@@ -33,6 +33,17 @@
 #include "goocanvasutils.h"
 
 
+typedef struct _GooCanvasImagePrivate GooCanvasImagePrivate;
+struct _GooCanvasImagePrivate {
+  gboolean scale_to_fit;
+};
+
+#define GOO_CANVAS_IMAGE_GET_PRIVATE(image)  \
+   (G_TYPE_INSTANCE_GET_PRIVATE ((image), GOO_TYPE_CANVAS_IMAGE, GooCanvasImagePrivate))
+#define GOO_CANVAS_IMAGE_MODEL_GET_PRIVATE(image)  \
+   (G_TYPE_INSTANCE_GET_PRIVATE ((image), GOO_TYPE_CANVAS_IMAGE_MODEL, GooCanvasImagePrivate))
+
+
 enum {
   PROP_0,
 
@@ -41,11 +52,13 @@ enum {
   PROP_Y,
   PROP_WIDTH,
   PROP_HEIGHT,
+  PROP_SCALE_TO_FIT,
 
   /* Convenience properties. */
   PROP_PIXBUF
 };
 
+static void goo_canvas_image_dispose      (GObject            *object);
 static void goo_canvas_image_finalize     (GObject            *object);
 static void canvas_item_interface_init    (GooCanvasItemIface *iface);
 static void goo_canvas_image_get_property (GObject            *object,
@@ -103,6 +116,13 @@ goo_canvas_image_install_common_properties (GObjectClass *gobject_class)
 							0.0, G_MAXDOUBLE, 0.0,
 							G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class, PROP_SCALE_TO_FIT, 
+                                   g_param_spec_boolean ("scale-to-fit",
+							 _("Scale To Fit"),
+							 _("If the image is scaled to fit the width and height settings"),
+							 FALSE,
+							 G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_PIXBUF,
 				   g_param_spec_object ("pixbuf",
 							_("Pixbuf"),
@@ -112,6 +132,27 @@ goo_canvas_image_install_common_properties (GObjectClass *gobject_class)
 }
 
 
+/* Gets the private data to use, from the model or from the item itself. */
+static GooCanvasImagePrivate*
+goo_canvas_image_get_private (gpointer object)
+{
+  GooCanvasItemSimple *simple;
+
+  if (GOO_IS_CANVAS_IMAGE (object))
+    {
+      simple = (GooCanvasItemSimple*) object;
+      if (simple->model)
+	return GOO_CANVAS_IMAGE_MODEL_GET_PRIVATE (simple->model);
+      else
+	return GOO_CANVAS_IMAGE_GET_PRIVATE (object);
+    }
+  else
+    {
+      return GOO_CANVAS_IMAGE_MODEL_GET_PRIVATE (object);
+    }
+}
+
+
 static void
 goo_canvas_image_init (GooCanvasImage *image)
 {
@@ -187,7 +228,7 @@ goo_canvas_image_new (GooCanvasItem *parent,
 
 
 static void
-goo_canvas_image_finalize (GObject *object)
+goo_canvas_image_dispose (GObject *object)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) object;
   GooCanvasImage *image = (GooCanvasImage*) object;
@@ -195,8 +236,23 @@ goo_canvas_image_finalize (GObject *object)
   if (!simple->model)
     {
       cairo_pattern_destroy (image->image_data->pattern);
-      g_slice_free (GooCanvasImageData, image->image_data);
+      image->image_data->pattern = NULL;
     }
+
+  G_OBJECT_CLASS (goo_canvas_image_parent_class)->dispose (object);
+}
+
+
+static void
+goo_canvas_image_finalize (GObject *object)
+{
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) object;
+  GooCanvasImage *image = (GooCanvasImage*) object;
+
+  /* Free our data if we didn't have a model. (If we had a model it would
+     have been reset in dispose() and simple_data will be NULL.) */
+  if (simple->simple_data)
+    g_slice_free (GooCanvasImageData, image->image_data);
   image->image_data = NULL;
 
   G_OBJECT_CLASS (goo_canvas_image_parent_class)->finalize (object);
@@ -210,6 +266,8 @@ goo_canvas_image_get_common_property (GObject              *object,
 				      GValue               *value,
 				      GParamSpec           *pspec)
 {
+  GooCanvasImagePrivate *priv = goo_canvas_image_get_private (object);
+
   switch (prop_id)
     {
     case PROP_PATTERN:
@@ -227,6 +285,9 @@ goo_canvas_image_get_common_property (GObject              *object,
     case PROP_HEIGHT:
       g_value_set_double (value, image_data->height);
       break;
+    case PROP_SCALE_TO_FIT:
+      g_value_set_boolean (value, priv->scale_to_fit);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -255,6 +316,7 @@ goo_canvas_image_set_common_property (GObject              *object,
 				      const GValue         *value,
 				      GParamSpec           *pspec)
 {
+  GooCanvasImagePrivate *priv = goo_canvas_image_get_private (object);
   GdkPixbuf *pixbuf;
 
   switch (prop_id)
@@ -276,12 +338,15 @@ goo_canvas_image_set_common_property (GObject              *object,
     case PROP_HEIGHT:
       image_data->height = g_value_get_double (value);
       break;
+    case PROP_SCALE_TO_FIT:
+      priv->scale_to_fit = g_value_get_boolean (value);
+      break;
     case PROP_PIXBUF:
       cairo_pattern_destroy (image_data->pattern);
       pixbuf = g_value_get_object (value);
-      image_data->pattern = goo_canvas_cairo_pattern_from_pixbuf (pixbuf);
-      image_data->width = gdk_pixbuf_get_width (pixbuf);
-      image_data->height = gdk_pixbuf_get_height (pixbuf);
+      image_data->pattern = pixbuf ? goo_canvas_cairo_pattern_from_pixbuf (pixbuf) : NULL;
+      image_data->width = pixbuf ? gdk_pixbuf_get_width (pixbuf) : 0;
+      image_data->height = pixbuf ? gdk_pixbuf_get_height (pixbuf) : 0;
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -311,21 +376,21 @@ goo_canvas_image_set_property (GObject              *object,
 }
 
 
-static GooCanvasItem*
-goo_canvas_image_get_item_at (GooCanvasItemSimple *simple,
-			      gdouble              x,
-			      gdouble              y,
-			      cairo_t             *cr,
-			      gboolean             is_pointer_event)
+static gboolean
+goo_canvas_image_is_item_at (GooCanvasItemSimple *simple,
+			     gdouble              x,
+			     gdouble              y,
+			     cairo_t             *cr,
+			     gboolean             is_pointer_event)
 {
   GooCanvasImage *image = (GooCanvasImage*) simple;
   GooCanvasImageData *image_data = image->image_data;
 
   if (x < image_data->x || (x > image_data->x + image_data->width)
       || y < image_data->y || (y > image_data->y + image_data->height))
-    return NULL;
+    return FALSE;
 
-  return (GooCanvasItem*) simple;
+  return TRUE;
 }
 
 
@@ -345,27 +410,51 @@ goo_canvas_image_update  (GooCanvasItemSimple  *simple,
 
 
 static void
-goo_canvas_image_paint (GooCanvasItemSimple *simple,
-			cairo_t             *cr,
-			GooCanvasBounds     *bounds)
+goo_canvas_image_paint (GooCanvasItemSimple   *simple,
+			cairo_t               *cr,
+			const GooCanvasBounds *bounds)
 {
+  GooCanvasImagePrivate *priv = goo_canvas_image_get_private (simple);
   GooCanvasImage *image = (GooCanvasImage*) simple;
   GooCanvasImageData *image_data = image->image_data;
-  cairo_matrix_t matrix;
+  cairo_matrix_t matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_surface_t *surface;
+  gdouble width, height;
 
   if (!image_data->pattern)
     return;
 
+#if 1
+  if (priv->scale_to_fit)
+    {
+      if (cairo_pattern_get_surface (image_data->pattern, &surface)
+	  == CAIRO_STATUS_SUCCESS
+	  && cairo_surface_get_type (surface) == CAIRO_SURFACE_TYPE_IMAGE)
+	{
+	  width = cairo_image_surface_get_width (surface);
+	  height = cairo_image_surface_get_height (surface);
+	  cairo_matrix_scale (&matrix, width / image_data->width,
+			      height / image_data->height);
+	}
+    }
+
+  cairo_matrix_translate (&matrix, -image_data->x, -image_data->y);
+
+  cairo_pattern_set_matrix (image_data->pattern, &matrix);
   goo_canvas_style_set_fill_options (simple->simple_data->style, cr);
   cairo_set_source (cr, image_data->pattern);
-  cairo_matrix_init_translate (&matrix, -image_data->x, -image_data->y);
-  cairo_pattern_set_matrix (image_data->pattern, &matrix);
   cairo_rectangle (cr, image_data->x, image_data->y,
 		   image_data->width, image_data->height);
   cairo_fill (cr);
-
-  /* Using cairo_paint() is much slower, so I guess we shouldn't. */
-  /*cairo_paint (cr);*/
+#else
+  /* Using cairo_paint() used to be much slower than cairo_fill(), though
+     they seem similar now. I'm not sure if it matters which we use. */
+  cairo_matrix_init_translate (&matrix, -image_data->x, -image_data->y);
+  cairo_pattern_set_matrix (image_data->pattern, &matrix);
+  goo_canvas_style_set_fill_options (simple->simple_data->style, cr);
+  cairo_set_source (cr, image_data->pattern);
+  cairo_paint (cr);
+#endif
 }
 
 
@@ -405,6 +494,9 @@ goo_canvas_image_class_init (GooCanvasImageClass *klass)
   GObjectClass *gobject_class = (GObjectClass*) klass;
   GooCanvasItemSimpleClass *simple_class = (GooCanvasItemSimpleClass*) klass;
 
+  g_type_class_add_private (gobject_class, sizeof (GooCanvasImagePrivate));
+
+  gobject_class->dispose  = goo_canvas_image_dispose;
   gobject_class->finalize = goo_canvas_image_finalize;
 
   gobject_class->get_property = goo_canvas_image_get_property;
@@ -412,7 +504,7 @@ goo_canvas_image_class_init (GooCanvasImageClass *klass)
 
   simple_class->simple_update      = goo_canvas_image_update;
   simple_class->simple_paint       = goo_canvas_image_paint;
-  simple_class->simple_get_item_at = goo_canvas_image_get_item_at;
+  simple_class->simple_is_item_at  = goo_canvas_image_is_item_at;
 
   goo_canvas_image_install_common_properties (gobject_class);
 }
@@ -444,7 +536,7 @@ goo_canvas_image_class_init (GooCanvasImageClass *klass)
  */
 
 static void item_model_interface_init (GooCanvasItemModelIface *iface);
-static void goo_canvas_image_model_finalize     (GObject            *object);
+static void goo_canvas_image_model_dispose      (GObject            *object);
 static void goo_canvas_image_model_get_property (GObject            *object,
 						 guint               param_id,
 						 GValue             *value,
@@ -465,7 +557,9 @@ goo_canvas_image_model_class_init (GooCanvasImageModelClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass*) klass;
 
-  gobject_class->finalize     = goo_canvas_image_model_finalize;
+  g_type_class_add_private (gobject_class, sizeof (GooCanvasImagePrivate));
+
+  gobject_class->dispose      = goo_canvas_image_model_dispose;
 
   gobject_class->get_property = goo_canvas_image_model_get_property;
   gobject_class->set_property = goo_canvas_image_model_set_property;
@@ -549,13 +643,14 @@ goo_canvas_image_model_new (GooCanvasItemModel *parent,
 
 
 static void
-goo_canvas_image_model_finalize (GObject *object)
+goo_canvas_image_model_dispose (GObject *object)
 {
   GooCanvasImageModel *imodel = (GooCanvasImageModel*) object;
 
   cairo_pattern_destroy (imodel->image_data.pattern);
+  imodel->image_data.pattern = NULL;
 
-  G_OBJECT_CLASS (goo_canvas_image_model_parent_class)->finalize (object);
+  G_OBJECT_CLASS (goo_canvas_image_model_parent_class)->dispose (object);
 }
 
 
